/*---------------------------------------------------------------------------*\
   =========                 |
\\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
\\    /   O peration     |
\\  /    A nd           | Copyright (C) 2011-2015 OpenFOAM Foundation
\\\\\\/     M anipulation  |
   -------------------------------------------------------------------------------
   License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

   Application
    reactingTwoPhaseEulerFoam

   Description
    Solver for a system of 2 compressible fluid phases with a common pressure,
    but otherwise separate properties. The type of phase model is run time
    selectable and can optionally represent multiple species and in-phase
    reactions. The phase system is also run time selectable and can optionally
    represent different types of momentun, heat and mass transfer.

\*---------------------------------------------------------------------------*/

#include "fvCFD.H"
#include "twoPhaseSystem.H"
#include "phaseCompressibleTurbulenceModel.H"
#include "fixedFluxPressureFvPatchScalarField.H"
#include "pimpleControl.H"
#include "localEulerDdtScheme.H"
#include "fvcSmooth.H"
#include "patchToPatchInterpolation.H" // for patchToPatch
#include "regionProperties.H" // for regions/domains
#include "OFstream.H" // for writing files etc.
#include "basicKinematicCloud.H" // from ALopez

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //





int main(int argc, char *argv[])
{
    #include "setRootCase.H"
    #include "createTime.H"

        regionProperties rp(runTime); //from chtMultiRegionSimpleFoam

    #include "createMesh.H"
    #include "createFluidMeshes.H" //from chtMultiRegionFoam
    #include "createSolidMeshes.H" //from chtMultiRegionFoam

        pimpleControl pimple(mesh);

    #include "createTimeControls.H"
    #include "createRDeltaT.H"
    #include "createFields.H"

        if (!LTS)
        {
        #include "CourantNo.H"
        #include "setInitialDeltaT.H"
        }

        Switch faceMomentum
        (
                pimple.dict().lookupOrDefault<Switch>("faceMomentum", false)
        );

        Switch implicitPhasePressure
        (
                mesh.solverDict(alpha1.name()).lookupOrDefault<Switch>
                (
                        "implicitPhasePressure", false
                )
        );

    #include "pUf/createDDtU.H"

        // * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

        Info<< "\nStarting time loop\n" << endl;

        label master = mesh.boundaryMesh().findPatchID("baffleFace1D_master"); //added by me
        label slave = mesh.boundaryMesh().findPatchID("baffleFace1D_slave"); //added by me

        while (runTime.run())
        {
        #include "readTimeControls.H"

                if (LTS)
                {
            #include "setRDeltaT.H"
                }
                else
                {
            #include "CourantNos.H"
            #include "setDeltaT.H"
                }

                runTime++;
                Info<< "Time = " << runTime.timeName() << nl << endl;

                // --- Pressure-velocity PIMPLE corrector loop
                while (pimple.loop())
                {
                        forAll(fluidRegions, i)
                        {
                                Info<< "\nSolving for first region "
                                    << fluidRegions[i].name() << endl;

                                fluid.solve();
                                fluid.correct();

                #include "YEqns.H"

                                if (faceMomentum)
                                {
                #include "pUf/UEqns.H"
                #include "EEqns.H"
                #include "pUf/pEqn.H"
                #include "pUf/DDtU.H"
                                }
                                else
                                {
                #include "pU/UEqns.H"
                #include "EEqns.H"
                #include "pU/pEqn.H"
                                }

                                fluid.correctKinematics();

                                if (pimple.turbCorr())
                                {
                                        fluid.correctTurbulence();
                                }

                        }
                #include "interpolateMasterSlave.H"

                vectorField normalSlaveVector = mesh.Sf().boundaryField()[slave];
                scalarField dotProduct = U1.boundaryField()[master] & normalSlaveVector;
                scalarField uNormal = dotProduct/mag(normalSlaveVector);

                vectorField centres = mesh.C().boundaryField()[slave];

                const fileName& fName(mesh.time().constant()/"kinematicLookupTableInjection");
                OFstream os(fName);
                os << "/*--------------------------------*- C++ -*----------------------------------* " << endl;
                os <<" | =========                 |                                                |\n";
                os <<" | \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox           |\n";
                os <<" |  \\    /   O peration     | Version:  3.0.x                                 |\n";
                os <<" |   \\  /    A nd           | Web:      www.OpenFOAM.org                      |\n";
                os <<" |    \\/     M anipulation  |                                                 |\n";
                os << "*---------------------------------------------------------------------------*/ //" << endl;
                os <<"FoamFile\n{\n";
                os <<"    version     2.0;\n";
                os <<"    format      ascii;\n";
                os <<"    class       dictionary;\n";
                os <<"    location    " "constant" ";\n";
                os <<"    object      kinematicLookupTableInjection;\n}\n";
                os <<"// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //\n\n";
                os <<"/* (x y z) (u v w) d rho mDot numParcels  \n";
                os <<"     where: \n";
                os <<"   x, y, z = global cartesian co-ordinates [m] \n";
                os <<"   u, v, w = global cartesian velocity components [m/s] \n";
                os <<"   d       = diameter [m] \n";
                os <<"   rho     = density [kg/m3] \n";
                os <<"   mDot    = mass flow rate [kg/m3] \n";
                os <<"   numParcels = number of Parcels  \n";

                os <<"   Dictionary for the KinematicLookupTableInjection */ \n";
                os << "( " << endl;
                forAll(interpolatedInletU2, i)      // Was U1
                {
                        os << centres[i] << " " << U1.boundaryField()[slave][i]
                        << " " << 55e-6 << " " << 2750 << " " << 0.005 << " "
                           << floor ((alpha1.boundaryField()[slave][i]*(mag(normalSlaveVector[i]))*uNormal[i])/((8.71e-14)*3*(-1)*5000))
                            << endl;

                }
                os << "//The end"<< endl;
                os << ");"<< endl;

                        forAll(solidRegions, i)
                        {
                            argList::addOption // from ALopez
                            (
                                    "cloudName",
                                    "name",
                                    "specify alternative cloud name. default is 'kinematicCloud'"
                            );


                        alpha1.boundaryField()[slave] == 0; // added by me

                                Info<< "\nSolving for second region "
                                    << solidRegions[i].name() << endl;

                                fluid.solve();
                                fluid.correct();

                    #include "YEqns.H"
                                if (faceMomentum)
                                {
                    #include "pUf/UEqns.H"
                    #include "EEqns.H"
                    #include "pUf/pEqn.H"
                    #include "pUf/DDtU.H"
                                }

                                else
                                {
                    #include "pU/UEqns.H"
                    #include "EEqns.H"
                    #include "pU/pEqn.H"
                                }

                                fluid.correctKinematics();

                                if (pimple.turbCorr())
                                {
                                        fluid.correctTurbulence();
                                }

                        }

                    #include "interpolateSlaveMaster.H"

                    Info<< "Evolving " << kinematicCloud.name() << endl; // from ALopez
                    kinematicCloud.evolve();

                }

                runTime.write();

                Info<< "ExecutionTime = "
                    << runTime.elapsedCpuTime()
                    << " s\n\n" << endl;
        }

        Info<< "End\n" << endl;

        return 0;
}


// ************************************************************************* //
